@@ -30,6 +30,8 @@ import volatility.plugins.common as common
 import volatility.utils as utils
 import volatility.win32.tasks as tasks
 import volatility.debug as debug
+from volatility.renderers import TreeGrid
+from volatility.renderers.basic import Address, Hex
 
 MAX_HISTORY_DEFAULT = 50
 
@@ -643,7 +645,7 @@ class CmdScan(common.AbstractWindowsCommand):
 
         for task in self.cmdhistory_process_filter(addr_space):
             process_space = task.get_process_address_space()
-            for found in task.search_process_memory([srch_pattern]):
+            for found in task.search_process_memory([srch_pattern], vad_filter = lambda x: x.Length < 0x40000000):
 
                 hist = obj.Object("_COMMAND_HISTORY",
                         vm = process_space,
@@ -653,6 +655,59 @@ class CmdScan(common.AbstractWindowsCommand):
                 if hist.is_valid(max_history = MAX_HISTORY):
                     yield task, hist
 
+    def unified_output(self, data):
+        return TreeGrid([("Process", str),
+                         ("PID", int),
+                         ("History Offset", Address),
+                         ("Application", str),
+                         ("Flags", str),
+                         ("Command Count", int),
+                         ("Last Added", str),
+                         ("Last Displayed", str),
+                         ("First Command", str),
+                         ("Command Count Max", int),
+                         ("Handle", int),
+                         ("Command Number", int),
+                         ("Command Offset", Address),
+                         ("Command", str)],
+                        self.generator(data))
+
+    def generator(self, data):
+
+        for task, hist in data:
+
+            # If the _COMMAND_HISTORY is in use, we would only take 
+            # hist.CommandCount but since we're brute forcing, try the 
+            # maximum and hope that some slots were not overwritten 
+            # or zero-ed out. 
+            pointers = obj.Object("Array", targetType = "address",
+                        count = hist.CommandCountMax,
+                        offset = hist.obj_offset +
+                        hist.obj_vm.profile.get_obj_offset("_COMMAND_HISTORY", "CommandBucket"),
+                        vm = hist.obj_vm)
+
+            values = [  str(task.ImageFileName),
+                        int(task.UniqueProcessId),
+                        Address(hist.obj_offset),
+                        str(hist.Application.dereference()),
+                        str(hist.Flags),
+                        int(hist.CommandCount),
+                        str(hist.LastAdded),
+                        str(hist.LastDisplayed),
+                        str(hist.FirstCommand),
+                        int(hist.CommandCountMax),
+                        int(hist.ProcessHandle),
+                     ]
+
+            for i, p in enumerate(pointers):
+                cmd = p.dereference_as("_COMMAND")
+                if cmd and str(cmd.Cmd):
+                    yield (0, values + [
+                                    int(i),
+                                    Address(cmd.obj_offset),
+                                    str(cmd.Cmd) ])
+
+
     def render_text(self, outfd, data):
 
         for task, hist in data:
@@ -705,7 +760,7 @@ class Consoles(CmdScan):
         srch_pattern = chr(self._config.MAX_HISTORY) + "\x00"
 
         for task in self.cmdhistory_process_filter(addr_space):
-            for found in task.search_process_memory([srch_pattern]):
+            for found in task.search_process_memory([srch_pattern], vad_filter = lambda x: x.Length < 0x40000000):
 
                 console = obj.Object("_CONSOLE_INFORMATION",
                     offset = found -
@@ -728,6 +783,183 @@ class Consoles(CmdScan):
 
                 yield task, console
 
+    def unified_output(self, data):
+        return TreeGrid([('Console Process', str),
+                         ('Console PID', int),
+                         ('Console ID', int),
+                         ('Command History Size', int),
+                         ('History Buffer Count', int),
+                         ('History Buffer Max', int),
+                         ('OriginalTitle', str),
+                         ('Title', str),
+                         ('Attached Process Name', str),
+                         ('Attached Process PID', int),
+                         ('Attached Process Handle', int),
+                         ('Command History ID', int),
+                         ('Command History Applications', str),
+                         ('Command History Flags', str),
+                         ('Command History Count', int),
+                         ('Command History Last Added', str),
+                         ('Command History Last Displayed', str),
+                         ('Command History First Command', str),
+                         ('Command History Command Count Max', int),
+                         ('Command History Process Handle', int),
+                         ('Command History Command Number', int),
+                         ('Command History Command Offset', Address),
+                         ('Command History Command String', str),
+                         ('EXE Alias', str),
+                         ('EXE Alias Source', str),
+                         ('EXE Alias Target', str),
+                         ('Screen ID', str),
+                         ('Screen X', int),
+                         ('Screen Y', int),
+                         ('Screen Dump', str)],
+                        self.generator(data))
+
+    def _get_values(self, task, console, process=None, console_proc=None,
+                    hist=None, hist_i=None, hist_cmd=None, exe_alias=None,
+                    screen=None):
+
+        # ('Console Process', str),
+        # ('Console PID', int),
+        # ('Console ID', int),
+        # ('Command History Size', int),
+        # ('History Buffer Count', int),
+        # ('History Buffer Max', int),
+        # ('OriginalTitle', str),
+        # ('Title', str),
+        v = [ str(task.ImageFileName),
+                 int(task.UniqueProcessId),
+                 int(console.obj_offset),
+                 int(console.CommandHistorySize),
+                 int(console.HistoryBufferCount),
+                 int(console.HistoryBufferMax),
+                 str(console.OriginalTitle.dereference()),
+                 str(console.Title.dereference()) ]
+
+        # ('Attached Process Name', str),
+        # ('Attached Process PID', int),
+        # ('Attached Process Handle', int),
+        if process is not None and console_proc is not None:
+            v.extend([ str(process.ImageFileName),
+                          int(process.UniqueProcessId),
+                          int(console_proc.ProcessHandle) ])
+        else:
+            v.extend([ "", -1, -1 ])
+
+        # ('Command History ID', int),
+        # ('Command History Applications', str),
+        # ('Command History Flags', str),
+        # ('Command History Count', int),
+        # ('Command History Last Added', str),
+        # ('Command History Last Displayed', str),
+        # ('Command History First Command', str),
+        # ('Command History Command Count Max', int),
+        # ('Command History Process Handle', int),
+        # ('Command History Command Number', int),
+        # ('Command History Command Offset', Address),
+        # ('Command History Command String', str),
+        if hist is not None:
+            v.extend([
+                int(hist.obj_offset),
+                str(hist.Application.dereference()),
+                str(hist.Flags),
+                int(hist.CommandCount),
+                str(hist.LastAdded),
+                str(hist.LastDisplayed),
+                str(hist.FirstCommand),
+                int(hist.CommandCountMax),
+                int(hist.ProcessHandle) ])
+            if hist_i is None or hist_cmd is None:
+                v.extend([ -1, Address(-1), '' ])
+            else:
+                v.extend([
+                    int(hist_i),
+                    Address(hist_cmd.obj_offset),
+                    str(hist_cmd.Cmd) ])
+        else:
+            v.extend([
+                -1,
+                '',
+                '',
+                -1,
+                '',
+                '',
+                '',
+                -1,
+                -1,
+                -1,
+                Address(-1),
+                '' ])
+
+        # ('EXE Alias', str),
+        # ('EXE Alias Source', str),
+        # ('EXE Alias Target', str),
+        if exe_alias is not None:
+            v.extend([
+                str(exe_alias.ExeName.dereference()),
+                str(alias.Source.dereference()),
+                str(alias.Target.dereference()) ])
+        else:
+            v.extend([ '', '', '' ])
+
+        # ('Screen ID', str),
+        # ('Screen X', int),
+        # ('Screen Y', int),
+        # ('Screen Dump', str)],
+        if screen is not None:
+            v.extend([
+                str(screen.dereference()),
+                int(screen.ScreenX),
+                int(screen.ScreenY),
+                '\n'.join(screen.get_buffer()) ])
+        else:
+            v.extend([ '', -1, -1, '' ])
+
+        return v
+
+    def generator(self, data):
+
+        for task, console in data:
+
+            has_yielded = False
+
+            for console_proc in console.get_processes():
+                process = console_proc.reference_object_by_handle()
+                if process:
+                    has_yielded = True
+                    yield (0, self._get_values(task, console, process=process,
+                        console_proc=console_proc))
+
+            for hist in console.get_histories():
+                cmds_processed = False
+                for i, cmd in hist.get_commands():
+                    if cmd.Cmd:
+                        cmds_processed = True
+                        yield (0, self._get_values(task, console, hist=hist,
+                                hist_i=i, hist_cmd=cmd ))
+                    has_yielded = cmds_processed
+                if not cmds_processed:
+                    # Did not generate any commands, so generate basic history
+                    # information so that no information is dropped.
+                    has_yielded = True
+                    yield (0, self._get_values(task, console, hist=hist))
+
+            for exe_alias in console.get_exe_aliases():
+                for alias in exe_alias.get_aliases():
+                    has_yielded = True
+                    yield (0, self._get_values(task, console, exe_alias=alias))
+
+            for screen in console.get_screens():
+                has_yielded = True
+                yield (0, self._get_values(task, console, screen=screen))
+
+            # if we have not yet generated any information
+            if not has_yielded:
+                # generate at least basic console information
+                yield (0, self._get_values(task, console))
+
+
     def render_text(self, outfd, data):
 
         for task, console in data:
